﻿using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.IO;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

public partial class _Default : System.Web.UI.Page
{
	protected void Page_Load(object sender, EventArgs e)
	{
	}

	protected void Submit_Click(object sender, EventArgs e)
	{
		if (IsPostBack)
			HandleUploadedFile();
	}

	/// <summary>
	/// Method to handle the saving and resizing of the uploaded image
	/// </summary>
	private void HandleUploadedFile()
	{
		// get the root of the web site
		string root = Server.MapPath("~/");

		// clean up the path
		if (!root.EndsWith(@"\"))
			root += @"\";

		// make a folder to store the images in
		string fileDirectory = root + @"\Images\";

		// create the folder if it does not exist
		if (!System.IO.Directory.Exists(fileDirectory))
			System.IO.Directory.CreateDirectory(fileDirectory);

		// make a link to the new file
		string link = "<a href='Images/{0}' target='_blank'>{1}</a>{2}{3}";

		// loop through the file in the request
		for (int i = 0; i < Request.Files.Count; i++)
		{

			// get the file instance
			HttpPostedFile fi = Request.Files.Get(i);

			// create a byte array to store the file bytes
			byte[] fileBytes = new byte[fi.ContentLength];

			// fill the byte array
			using (System.IO.Stream stream = fi.InputStream)
			{
				stream.Read(fileBytes, 0, fi.ContentLength);
			}

			// create a random file name
			string fileName = Guid.NewGuid().ToString();

			// write the original file to the file system
			File.WriteAllBytes(fileDirectory + fileName + ".jpg", 
				fileBytes);

			litText.Text += string.Format(link, fileName + ".jpg", 
				fileName + " Original", "<br/>", "");

			// write the resized file to the file system
			File.WriteAllBytes(fileDirectory + fileName + "_small.jpg", 
				ResizeImageFile(fileBytes, 600));

			litText.Text += string.Format(link, fileName + "_small.jpg", 
				fileName + " Small", "<br/>", "<br/>");

			// cleanup
			litText.Visible = true;
			fileBytes = null;
		}
	}

	/// <summary>
	/// Resizes an image
	/// </summary>
	/// <param name="imageFile">the byte array of the file</param>
	/// <param name="targetSize">the target size of the file (may affect width or height) 
	/// depends on orientation of file (landscape or portrait)</param>
	/// <returns>Byte array containing the resized file</returns>
	private static byte[] ResizeImageFile(byte[] imageFile, int targetSize)
	{
		using (System.Drawing.Image oldImage = 
			System.Drawing.Image.FromStream(new MemoryStream(imageFile)))
		{
			Size newSize = CalculateDimensions(oldImage.Size, targetSize);

			using (Bitmap newImage = 
				new Bitmap(newSize.Width, 
					newSize.Height, PixelFormat.Format24bppRgb))
			{
				using (Graphics canvas = Graphics.FromImage(newImage))
				{
					canvas.SmoothingMode = SmoothingMode.AntiAlias;
					canvas.InterpolationMode = InterpolationMode.HighQualityBicubic;
					canvas.PixelOffsetMode = PixelOffsetMode.HighQuality;
					canvas.DrawImage(oldImage, 
						new Rectangle(new Point(0, 0), newSize));

					MemoryStream m = new MemoryStream();
					newImage.Save(m, ImageFormat.Jpeg);
					return m.GetBuffer();
				}
			}
		}
	}

	/// <summary>
	/// Calculates the new size of the image based on the target size
	/// </summary>
	/// <param name="oldSize">Is the size of the original file</param>
	/// <param name="targetSize">Is the target size of the resized file</param>
	/// <returns>The new size</returns>
	private static Size CalculateDimensions(Size oldSize, int targetSize)
	{
		Size newSize = new Size();
		if (oldSize.Height > oldSize.Width)
		{
			newSize.Width = 
				(int)(oldSize.Width * ((float)targetSize / (float)oldSize.Height));
			newSize.Height = targetSize;
		}
		else
		{
			newSize.Width = targetSize;
			newSize.Height = 
				(int)(oldSize.Height * ((float)targetSize / (float)oldSize.Width));
		}
		return newSize;
	}
}